@@ -1,5 +1,7 @@
 #!/bin/bash
 
+set -x
+
 source activate $CONDA_ENV
 
 # need to build with Anaconda compilers on osx, but they conflict with llvmdev... bootstap
@@ -13,12 +15,14 @@ if [[ $(uname) == Darwin ]]; then
   export PATH=${SRC_DIR}/bootstrap/bin:${PATH}
   CONDA_PREFIX=${SRC_DIR}/bootstrap \
     . ${SRC_DIR}/bootstrap/etc/conda/activate.d/*
-  export CONDA_BUILD_SYSROOT=${CONDA_BUILD_SYSROOT:-/opt/MacOSX${MACOSX_DEPLOYMENT_TARGET}.sdk}
+  SDKPATH=$(xcrun --show-sdk-path)
+  export CONDA_BUILD_SYSROOT=${CONDA_BUILD_SYSROOT:-${SDKPATH}}
   export CXXFLAGS=${CFLAGS}" -mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET}"
   export CFLAGS=${CFLAGS}" -mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET}"
   SYSROOT_DIR=${CONDA_BUILD_SYSROOT}
   CFLAG_SYSROOT="--sysroot ${SYSROOT_DIR}"
   ${LLVM_CONFIG} --version
+  export SDKROOT=${SDKPATH}
 fi
 
 if [ -n "$MACOSX_DEPLOYMENT_TARGET" ]; then
