@@ -489,10 +489,8 @@ LLVMPY_WriteCFG(LLVMValueRef Fval, const char **OutStr, int ShowInst) {
     Function *F  = unwrap<Function>(Fval);
     std::string buffer;
     raw_string_ostream stream(buffer);
-    // Note: The (const Function*)F is necessary to trigger the right behavior.
-    //       A non constant Function* will result in the instruction not
-    //       printed regardless of the value in the 3rd argument.
-    WriteGraph(stream, (const Function*)F, !ShowInst);
+    DOTFuncInfo CFGInfo(F, nullptr, nullptr, 0);
+    WriteGraph(stream, &CFGInfo, !ShowInst);
     *OutStr = LLVMPY_CreateString(stream.str().c_str());
 }
 
