@@ -2,9 +2,8 @@
 Tests for legacy llvmpy-compatibility APIs.
 """
 
-from __future__ import print_function, absolute_import
-
-from . import TestCase
+import unittest
+from llvmlite.tests import TestCase
 
 
 class TestMisc(TestCase):
@@ -13,8 +12,8 @@ class TestMisc(TestCase):
         """
         Sanity-check llvmpy APIs import correctly.
         """
-        from llvmlite.llvmpy.core import Constant, Type, Builder
-        from llvmlite.llvmpy.passes import create_pass_manager_builder
+        from llvmlite.llvmpy.core import Constant, Type, Builder  # noqa F401
+        from llvmlite.llvmpy.passes import create_pass_manager_builder  # noqa F401 E501
 
 
 if __name__ == '__main__':
